@@ -1,15 +1,15 @@
 package Net::DNS::Parameters;
 
 #
-# $Id: Parameters.pm 1561 2017-04-19 13:08:13Z willem $
+# $Id: Parameters.pm 1761 2020-01-01 11:58:34Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1761 $)[1];
 
 
 ################################################
 ##
 ##	Domain Name System (DNS) Parameters
-##	(last updated 2017-04-13)
+##	(last updated 2019-12-23)
 ##
 ################################################
 
@@ -26,23 +26,25 @@ our @EXPORT = qw(
 		opcodebyname opcodebyval
 		rcodebyname rcodebyval
 		ednsoptionbyname ednsoptionbyval
+		dsotypebyname dsotypebyval
 		);
 
 
 # Registry: DNS CLASSes
-our %classbyname = (
+my @classbyname = (
 	IN   => 1,						# RFC1035
 	CH   => 3,						# Chaosnet
 	HS   => 4,						# Hesiod
 	NONE => 254,						# RFC2136
 	ANY  => 255,						# RFC1035
 	);
-our %classbyval = reverse %classbyname;
-%classbyname = ( '*' => 255, %classbyname, map lc($_), %classbyname );
+our %classbyval = reverse( CLASS0 => 0, @classbyname );
+push @classbyname, map /^\d/ ? $_ : lc($_), @classbyname;
+our %classbyname = ( '*' => 255, @classbyname );
 
 
 # Registry: Resource Record (RR) TYPEs
-our %typebyname = (
+my @typebyname = (
 	A	   => 1,					# RFC1035
 	NS	   => 2,					# RFC1035
 	MD	   => 3,					# RFC1035
@@ -95,7 +97,7 @@ our %typebyname = (
 	NSEC3	   => 50,					# RFC5155
 	NSEC3PARAM => 51,					# RFC5155
 	TLSA	   => 52,					# RFC6698
-	SMIMEA	   => 53,					# RFC-ietf-dane-smime-16
+	SMIMEA	   => 53,					# RFC8162
 	HIP	   => 55,					# RFC8005
 	NINFO	   => 56,					#
 	RKEY	   => 57,					#
@@ -104,6 +106,7 @@ our %typebyname = (
 	CDNSKEY	   => 60,					# RFC7344
 	OPENPGPKEY => 61,					# RFC7929
 	CSYNC	   => 62,					# RFC7477
+	ZONEMD	   => 63,					# draft-wessels-dns-zone-digest
 	SPF	   => 99,					# RFC7208
 	UINFO	   => 100,					# IANA-Reserved
 	UID	   => 101,					# IANA-Reserved
@@ -121,31 +124,36 @@ our %typebyname = (
 	AXFR	   => 252,					# RFC1035 RFC5936
 	MAILB	   => 253,					# RFC1035
 	MAILA	   => 254,					# RFC1035
-	ANY	   => 255,					# RFC1035 RFC6895
+	ANY	   => 255,					# RFC1035 RFC6895 RFC8482
 	URI	   => 256,					# RFC7553
-	CAA	   => 257,					# RFC6844
+	CAA	   => 257,					# RFC8659
 	AVC	   => 258,					#
+	DOA	   => 259,					# draft-durand-doa-over-dns
+	AMTRELAY   => 260,					# RFC-ietf-mboned-driad-amt-discovery-13
 	TA	   => 32768,					# http://cameo.library.cmu.edu/ http://www.watson.org/~weiler/INI1999-19.pdf
-	DLV	   => 32769,					# RFC4431
+	DLV	   => 32769,					# RFC-ietf-dnsop-obsolete-dlv-02 RFC4431
 	);
-our %typebyval = reverse %typebyname;
-%typebyname = ( '*' => 255, %typebyname, map lc($_), %typebyname );
+our %typebyval = reverse( TYPE0 => 0, @typebyname );
+push @typebyname, map /^\d/ ? $_ : lc($_), @typebyname;
+our %typebyname = ( '*' => 255, @typebyname );
 
 
 # Registry: DNS OpCodes
-our %opcodebyname = (
+my @opcodebyname = (
 	QUERY  => 0,						# RFC1035
 	IQUERY => 1,						# RFC3425
 	STATUS => 2,						# RFC1035
 	NOTIFY => 4,						# RFC1996
 	UPDATE => 5,						# RFC2136
+	DSO    => 6,						# RFC8490
 	);
-our %opcodebyval = reverse %opcodebyname;
-%opcodebyname = ( NS_NOTIFY_OP => 4, %opcodebyname, map lc($_), %opcodebyname );
+our %opcodebyval = reverse @opcodebyname;
+push @opcodebyname, map /^\d/ ? $_ : lc($_), @opcodebyname;
+our %opcodebyname = ( NS_NOTIFY_OP => 4, @opcodebyname );
 
 
 # Registry: DNS RCODEs
-our %rcodebyname = (
+my @rcodebyname = (
 	NOERROR	  => 0,						# RFC1035
 	FORMERR	  => 1,						# RFC1035
 	SERVFAIL  => 2,						# RFC1035
@@ -158,6 +166,7 @@ our %rcodebyname = (
 	NOTAUTH	  => 9,						# RFC2136
 	NOTAUTH	  => 9,						# RFC2845
 	NOTZONE	  => 10,					# RFC2136
+	DSOTYPENI => 11,					# RFC8490
 	BADVERS	  => 16,					# RFC6891
 	BADSIG	  => 16,					# RFC2845
 	BADKEY	  => 17,					# RFC2845
@@ -168,13 +177,14 @@ our %rcodebyname = (
 	BADTRUNC  => 22,					# RFC4635
 	BADCOOKIE => 23,					# RFC7873
 	);
-our %rcodebyval = reverse( BADSIG => 16, %rcodebyname );
-%rcodebyname = ( %rcodebyname, map lc($_), %rcodebyname );
+our %rcodebyval = reverse( BADSIG => 16, @rcodebyname );
+push @rcodebyname, map /^\d/ ? $_ : lc($_), @rcodebyname;
+our %rcodebyname = @rcodebyname;
 
 
 # Registry: DNS EDNS0 Option Codes (OPT)
-our %ednsoptionbyname = (
-	LLQ		=> 1,					# http://files.dns-sd.org/draft-sekar-dns-llq.txt
+my @ednsoptionbyname = (
+	LLQ		=> 1,					# RFC-sekar-dns-llq-06
 	UL		=> 2,					# http://files.dns-sd.org/draft-sekar-dns-ul.txt
 	NSID		=> 3,					# RFC5001
 	DAU		=> 5,					# RFC6975
@@ -187,14 +197,17 @@ our %ednsoptionbyname = (
 	PADDING		=> 12,					# RFC7830
 	CHAIN		=> 13,					# RFC7901
 	'KEY-TAG'	=> 14,					# RFC8145
+	'CLIENT-TAG'	=> 16,					# draft-bellis-dnsop-edns-tags
+	'SERVER-TAG'	=> 17,					# draft-bellis-dnsop-edns-tags
 	DEVICEID	=> 26946,				# https://docs.umbrella.com/developer/networkdevices-api/identifying-dns-traffic2
 	);
-our %ednsoptionbyval = reverse %ednsoptionbyname;
-%ednsoptionbyname = ( %ednsoptionbyname, map lc($_), %ednsoptionbyname );
+our %ednsoptionbyval = reverse @ednsoptionbyname;
+push @ednsoptionbyname, map /^\d/ ? $_ : lc($_), @ednsoptionbyname;
+our %ednsoptionbyname = @ednsoptionbyname;
 
 
 # Registry: DNS Header Flags
-our %dnsflagbyname = (
+my @dnsflagbyname = (
 	AA => 0x0400,						# RFC1035
 	TC => 0x0200,						# RFC1035
 	RD => 0x0100,						# RFC1035
@@ -202,14 +215,31 @@ our %dnsflagbyname = (
 	AD => 0x0020,						# RFC4035 RFC6840
 	CD => 0x0010,						# RFC4035 RFC6840
 	);
-%dnsflagbyname = ( %dnsflagbyname, map lc($_), %dnsflagbyname );
+push @dnsflagbyname, map /^\d/ ? $_ : lc($_), @dnsflagbyname;
+our %dnsflagbyname = @dnsflagbyname;
 
 
 # Registry: EDNS Header Flags (16 bits)
-our %ednsflagbyname = (
+my @ednsflagbyname = (
 	DO => 0x8000,						# RFC4035 RFC3225 RFC6840
 	);
-%ednsflagbyname = ( %ednsflagbyname, map lc($_), %ednsflagbyname );
+push @ednsflagbyname, map /^\d/ ? $_ : lc($_), @ednsflagbyname;
+our %ednsflagbyname = @ednsflagbyname;
+
+
+# Registry: DSO Type Codes
+my @dsotypebyname = (
+	KEEPALIVE	  => 0x0001,				# RFC8490
+	RETRYDELAY	  => 0x0002,				# RFC8490
+	ENCRYPTIONPADDING => 0x0003,				# RFC8490
+	SUBSCRIBE	  => 0x0040,				# RFC-ietf-dnssd-push-25
+	PUSH		  => 0x0041,				# RFC-ietf-dnssd-push-25
+	UNSUBSCRIBE	  => 0x0042,				# RFC-ietf-dnssd-push-25
+	RECONFIRM	  => 0x0043,				# RFC-ietf-dnssd-push-25
+	);
+our %dsotypebyval = reverse @dsotypebyname;
+push @dsotypebyname, map /^\d/ ? $_ : lc($_), @dsotypebyname;
+our %dsotypebyname = @dsotypebyname;
 
 
 ########
@@ -220,11 +250,11 @@ sub classbyname {
 	my $name = shift;
 
 	$classbyname{$name} || $classbyname{uc $name} || do {
-		croak "unknown class $name" unless $name =~ m/^(CLASS)?(\d+)/i;
+		croak qq[unknown class "$name"] unless $name =~ m/^(CLASS)?(\d+)/i;
 		my $val = 0 + $2;
-		croak "classbyname( $name ) out of range" if $val > 0xffff;
+		croak qq[classbyname("$name") out of range] if $val > 0xffff;
 		return $val;
-			}
+	}
 }
 
 sub classbyval {
@@ -232,9 +262,9 @@ sub classbyval {
 
 	$classbyval{$val} || do {
 		$val += 0;
-		croak "classbyval( $val ) out of range" if $val > 0xffff;
+		croak qq[classbyval($val) out of range] if $val > 0xffff;
 		return "CLASS$val";
-			}
+	}
 }
 
 
@@ -244,12 +274,12 @@ sub typebyname {
 	$typebyname{$name} || do {
 		if ( $name =~ m/^(TYPE)?(\d+)/i ) {
 			my $val = 0 + $2;
-			croak "typebyname( $name ) out of range" if $val > 0xffff;
+			croak qq[typebyname("$name") out of range] if $val > 0xffff;
 			return $val;
 		}
-		_typespec("$name.RRNAME");
-		croak "unknown type $name" unless $typebyname{uc $name};
-			}
+		_typespec("$name.RRNAME") unless $typebyname{uc $name};
+		return $typebyname{uc $name} || croak qq[unknown type "$name"];
+	}
 }
 
 sub typebyval {
@@ -257,11 +287,11 @@ sub typebyval {
 
 	$typebyval{$val} || do {
 		$val += 0;
-		croak "typebyval( $val ) out of range" if $val > 0xffff;
+		croak qq[typebyval($val) out of range] if $val > 0xffff;
 		$typebyval{$val} = "TYPE$val";
 		_typespec("$val.RRTYPE");
 		return $typebyval{$val};
-			}
+	}
 }
 
 
@@ -269,7 +299,7 @@ sub opcodebyname {
 	my $arg = shift;
 	return $opcodebyname{$arg} if defined $opcodebyname{$arg};
 	return 0 + $arg if $arg =~ /^\d/;
-	croak "unknown opcode $arg";
+	croak qq[unknown opcode "$arg"];
 }
 
 sub opcodebyval {
@@ -282,7 +312,7 @@ sub rcodebyname {
 	my $arg = shift;
 	return $rcodebyname{$arg} if defined $rcodebyname{$arg};
 	return 0 + $arg if $arg =~ /^\d/;
-	croak "unknown rcode $arg";
+	croak qq[unknown rcode "$arg"];
 }
 
 sub rcodebyval {
@@ -295,7 +325,7 @@ sub ednsoptionbyname {
 	my $arg = shift;
 	return $ednsoptionbyname{$arg} if defined $ednsoptionbyname{$arg};
 	return 0 + $arg if $arg =~ /^\d/;
-	croak "unknown option $arg";
+	croak qq[unknown option "$arg"];
 }
 
 sub ednsoptionbyval {
@@ -304,49 +334,59 @@ sub ednsoptionbyval {
 }
 
 
-our $DNSEXTLANG = 'ARPA.';		## draft-levine-dnsextlang
+sub dsotypebyname {
+	my $arg = shift;
+	return $dsotypebyname{$arg} if defined $dsotypebyname{$arg};
+	return 0 + $arg if $arg =~ /^\d/;
+	croak qq[unknown DSO type "$arg"];
+}
 
-use constant DNSEXTLANG => defined eval <<'END';
-	die 'preempt failure' if $^O =~ /cygwin|MSWin32/i;
-	require IO::File;
-	local $SIG{__WARN__} = sub { };
-	new IO::File('RRTYPEgen |') or die $!;
-END
+sub dsotypebyval {
+	my $val = shift;
+	$dsotypebyval{$val} || return $val;
+}
 
 
 sub register {				## register( 'TOY', 1234 )	(NOT part of published API)
 	my ( $mnemonic, $rrtype ) = map uc($_), @_;		# uncoverable pod
 	$rrtype = rand(255) + 65280 unless $rrtype;
-	for ( typebyval $rrtype = int($rrtype) ) {
-		croak "'$mnemonic' is a CLASS identifier" if $classbyname{$mnemonic};
+	for ( typebyval( $rrtype = int $rrtype ) ) {
+		croak qq["$mnemonic" is a CLASS identifier] if $classbyname{$mnemonic};
 		return $rrtype if /^$mnemonic$/;    # duplicate registration
-		croak "'$mnemonic' conflicts with TYPE$rrtype ($_)" unless /^TYPE\d+$/;
+		croak qq["$mnemonic" conflicts with TYPE$rrtype ($_)] unless /^TYPE\d+$/;
 		my $known = $typebyname{$mnemonic};
-		croak "'$mnemonic' conflicts with TYPE$known" if $known;
+		croak qq["$mnemonic" conflicts with TYPE$known] if $known;
 	}
 	$typebyval{$rrtype} = $mnemonic;
 	return $typebyname{$mnemonic} = $rrtype;
 }
 
 
+use constant EXTLANG => defined eval 'require Net::DNS::Extlang';
+
+our $DNSEXTLANG = EXTLANG ? eval 'Net::DNS::Extlang->new()->domain' : undef;
+
 sub _typespec {				## draft-levine-dnsextlang
-	eval <<'END' if DNSEXTLANG;
+	eval <<'END' if EXTLANG && $DNSEXTLANG;
 	my ($node) = @_;
+
 	require Net::DNS::Resolver;
-	my $resolver = new Net::DNS::Resolver;
-	my $response = $resolver->send( "$node.$DNSEXTLANG", 'TXT' );
+	my $resolver = new Net::DNS::Resolver() || return;
+	my $response = $resolver->send( "$node.$DNSEXTLANG", 'TXT' ) || return;
 
 	foreach my $txt ( grep $_->type eq 'TXT', $response->answer ) {
 		my @stanza = $txt->txtdata;
-		my ( $tag, $identifier ) = @stanza;
+		my ( $tag, $identifier, @attribute ) = @stanza;
 		next unless defined($tag) && $tag =~ /^RRTYPE=\d+$/;
-		register( split /[:\s]/, $identifier );
+		register( $1, $2 ) if $identifier =~ /^(\w+):(\d+)\W*/;
 		return unless defined wantarray;
-		require 5.008009;				# support for reference in @INC
-		my @arg = map { s/\s.*$//; qq("$_") } @stanza;	# strip descriptive text
-		return new IO::File("RRTYPEgen @arg |");
+
+		my $extobj = new Net::DNS::Extlang();
+		my $recipe = $extobj->xlstorerecord( $identifier, @attribute );
+		my @source = split /\n/, $extobj->compilerr($recipe);
+		return sub { defined( $_ = shift @source ) };
 	}
-	return undef;
+	return;
 END
 }
 
@@ -357,7 +397,7 @@ __END__
 
 =head1 NAME
 
-    Net::DNS::Parameters - DNS parameter assignments
+Net::DNS::Parameters - DNS parameter assignments
 
 
 =head1 SYNOPSIS
@@ -374,12 +414,12 @@ maintained and published by IANA.
 
 =head1 FUNCTIONS
 
-=head2 classbyname, typebyname, opcodebyname, rcodebyname, ednsoptionbyname
+=head2 classbyname, typebyname, opcodebyname, rcodebyname, ednsoptionbyname, dsotypebyname
 
 Access functions which return the numerical code corresponding to
 the given mnemonic.
 
-=head2 classbyval, typebyval, opcodebyval, rcodebyval, ednsoptionbyval
+=head2 classbyval, typebyval, opcodebyval, rcodebyval, ednsoptionbyval, dsotypebyval
 
 Access functions which return the canonical mnemonic corresponding to
 the given numerical code.
