@@ -1,9 +1,9 @@
 package Net::DNS::RR::RRSIG;
 
 #
-# $Id: RRSIG.pm 1561 2017-04-19 13:08:13Z willem $
+# $Id: RRSIG.pm 1754 2019-08-19 14:12:28Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1561 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1754 $)[1];
 
 
 use strict;
@@ -29,16 +29,27 @@ use constant DEBUG => 0;
 
 use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';
 
-use constant PRIVATE => defined eval 'require Net::DNS::SEC::Private';
+# IMPORTANT: Downstream distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
+use constant USESEC => defined $INC{'Net/DNS/SEC.pm'};		# Discover how we got here, without exposing any crypto
+								# Discourage static code analysers and casual greppers
+use constant DNSSEC => USESEC && defined eval join '', qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);
 
-use constant DSA => defined eval 'require Net::DNS::SEC::DSA';
-use constant RSA => defined eval 'require Net::DNS::SEC::RSA';
+my @index;
+if (DNSSEC) {
+	my $key = new Net::DNS::RR( type => 'DNSKEY', key => 'AwEAAQ==' );
+	foreach my $class ( map "Net::DNS::SEC::$_", qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
+		my @algorithms = eval join '', qw(r e q u i r e), " $class; $class->_index";	# 1.14 API
+		@algorithms = grep eval { $key->algorithm($_); $class->verify( '', $key, '' ); 1 }, ( 1 .. 16 )
+				unless scalar(@algorithms);	# Grotesquely inefficient; but need to support older API
+		push @index, map( ( $_ => $class ), @algorithms );
+	}
+}
 
-use constant ECDSA => defined eval 'require Net::DNS::SEC::ECDSA';
-use constant EdDSA => defined eval 'require Net::DNS::SEC::EdDSA';
-use constant GOST  => defined eval 'require Net::DNS::SEC::ECCGOST';
+my %DNSSEC_verify = @index;
+my %DNSSEC_siggen = @index;
 
-use constant DNSSEC => PRIVATE && ( RSA || DSA || ECDSA || EdDSA || GOST );
+my @deprecated = ( 1, 3, 6, 12 );				# RFC8624
+delete @DNSSEC_siggen{@deprecated};
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
@@ -57,7 +68,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $signame = $self->{signame} || return '';
+	my $signame = $self->{signame};
 	pack 'n C2 N3 n a* a*', @{$self}{@field}, $signame->canonical, $self->sigbin;
 }
 
@@ -65,9 +76,9 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $signame = $self->{signame} || return '';
-	my @sig64 = split /\s+/, encode_base64( $self->sigbin );
-	my @rdata = ( map( $self->$_, @field ), $signame->string, @sig64 );
+	my $signame = $self->{signame};
+	my @sig64   = split /\s+/, encode_base64( $self->sigbin );
+	my @rdata   = ( map( $self->$_, @field ), $signame->string, @sig64 );
 }
 
 
@@ -91,7 +102,7 @@ sub _defaults {				## specify RR attribute default values
 #
 {
 	my @algbyname = (
-		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8087]
+		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
 		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
 		'DH'		     => 2,			# [RFC2539]
 		'DSA'		     => 3,			# [RFC3755][RFC2536]
@@ -117,24 +128,16 @@ sub _defaults {				## specify RR attribute default values
 
 	my %algbyval = reverse @algbyname;
 
-	my $map = sub {
-		my $arg = shift;
-		unless ( $arg =~ /^\d/ ) {
-			$arg =~ s/[^A-Za-z0-9]//g;		# synthetic key
-			return uc $arg;
-		}
-		my @map = ( $arg, "$arg" => $arg );		# also accept number
-	};
-
-	my %algbyname = map &$map($_), @algbyname;
+	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
+	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
 		my $arg = shift;
 		my $key = uc $arg;				# synthetic key
-		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
+		$key =~ s/[\W_]//g;				# strip non-alphanumerics
 		my $val = $algbyname{$key};
 		return $val if defined $val;
-		return $key =~ /^\d/ ? $arg : croak "unknown algorithm $arg";
+		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm "$arg"];
 	}
 
 	sub _algbyval {
@@ -144,27 +147,6 @@ sub _defaults {				## specify RR attribute default values
 }
 
 
-my $RSA	  = RSA	  ? 'Net::DNS::SEC::RSA'     : 0;
-my $DSA	  = DSA	  ? 'Net::DNS::SEC::DSA'     : 0;
-my $ECDSA = ECDSA ? 'Net::DNS::SEC::ECDSA'   : 0;
-my $EdDSA = EdDSA ? 'Net::DNS::SEC::EdDSA'   : 0;
-my $GOST  = GOST  ? 'Net::DNS::SEC::ECCGOST' : 0;
-
-my %SEC = (
-	3  => $DSA,
-	5  => $RSA,
-	6  => $DSA,
-	7  => $RSA,
-	8  => $RSA,
-	10 => $RSA,
-	12 => $GOST,
-	13 => $ECDSA,
-	14 => $ECDSA,
-	15 => $EdDSA,
-	16 => $EdDSA,
-	);
-
-
 sub typecovered {
 	my $self = shift;
 	$self->{typecovered} = typebyname(shift) if scalar @_;
@@ -248,9 +230,8 @@ sub signame {
 
 sub sig {
 	my $self = shift;
-
-	$self->sigbin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	MIME::Base64::encode( $self->sigbin(), "" ) if defined wantarray;
+	return MIME::Base64::encode( $self->sigbin(), "" ) unless scalar @_;
+	$self->sigbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -267,7 +248,7 @@ sub signature { &sig; }
 
 sub create {
 	unless (DNSSEC) {
-		croak 'Net::DNS::SEC support not available';
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
 	} else {
 		my ( $class, $rrsetref, $priv_key, %etc ) = @_;
 
@@ -277,8 +258,7 @@ sub create {
 
 		# All the TTLs need to be the same in the data RRset.
 		my $ttl = $RR->ttl;
-		my @ttl = grep $_->ttl != $ttl, @$rrsetref;
-		croak 'RRs in RRset do not have same TTL' if scalar @ttl;
+		croak 'RRs in RRset do not have same TTL' if grep $_->ttl != $ttl, @$rrsetref;
 
 		my $private = ref($priv_key) ? $priv_key : Net::DNS::SEC::Private->new($priv_key);
 		croak 'unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
@@ -306,7 +286,8 @@ sub create {
 		$self->{sigexpiration} = $self->{siginception} + $self->{sigval}
 				unless $self->{sigexpiration};
 
-		$self->_CreateSig( $self->_CreateSigData($rrsetref), $private );
+		my $sigdata = $self->_CreateSigData($rrsetref);
+		$self->_CreateSig( $sigdata, $private );
 		return $self;
 	}
 }
@@ -318,10 +299,11 @@ sub verify {
 
 	# $rrsetref must be a reference to an array of RR objects.
 
-	# $keyref is either a key object or a reference to an array
-	# of key objects.
+	# $keyref is either a key object or a reference to an array of key objects.
 
-	if (DNSSEC) {
+	unless (DNSSEC) {
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
+	} else {
 		my ( $self, $rrsetref, $keyref ) = @_;
 
 		croak '$keyref argument is scalar or undefined' unless ref($keyref);
@@ -329,22 +311,19 @@ sub verify {
 		print '$keyref argument is ', ref($keyref), "\n" if DEBUG;
 		if ( ref($keyref) eq "ARRAY" ) {
 
-			#  We will recurse for each key that matches algorithm and key-id
-			#  and return when there is a successful verification.
-			#  If not, we will continue so that we can survive key-id collision.
-			#  The downside of this is that the error string only matches the
-			#  last error.
+			#  We will iterate over the supplied key list and
+			#  return when there is a successful verification.
+			#  If not, continue so that we survive key-id collision.
 
 			print "Iterating over ", scalar(@$keyref), " keys\n" if DEBUG;
 			my @error;
-			my $i;
 			foreach my $keyrr (@$keyref) {
 				my $result = $self->verify( $rrsetref, $keyrr );
 				return $result if $result;
 				my $error = $self->{vrfyerrstr};
-				$i++;
-				push @error, "key $i: $error";
-				print "key $i: $error\n" if DEBUG;
+				my $keyid = $keyrr->keytag;
+				push @error, "key $keyid: $error";
+				print "key $keyid: $error\n" if DEBUG;
 				next;
 			}
 
@@ -382,7 +361,8 @@ sub verify {
 			return 0;
 		}
 
-		$self->_VerifySig( $self->_CreateSigData($rrsetref), $keyref ) || return 0;
+		my $sigdata = $self->_CreateSigData($rrsetref);
+		$self->_VerifySig( $sigdata, $keyref ) || return 0;
 
 		# time to do some time checking.
 		my $t = time;
@@ -493,9 +473,6 @@ sub _CreateSigData {
 
 		print "_CreateSigData\n" if DEBUG;
 
-		croak 'SIG0 using RRSIG not permitted' unless ref($rrsetref);
-
-		my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 		my $sigdata = pack 'n C2 N3 n a*', @{$self}{@field}, $self->{signame}->canonical;
 		print "\npreamble\t", unpack( 'H*', $sigdata ), "\n" if DEBUG;
 
@@ -503,16 +480,16 @@ sub _CreateSigData {
 		my $limit = $self->{labels};
 		my @label = $owner->_wire;
 		shift @label while scalar @label > $limit;
-		my $wild = bless {label => \@label}, ref($owner);    # DIY to avoid wrecking name cache
+		my $wild   = bless {label => \@label}, ref($owner);    # DIY to avoid wrecking name cache
 		my $suffix = $wild->canonical;
 		unshift @label, chr(42);			# asterisk
 
-		my @RR	  = map bless( {%$_}, ref($_) ), @$rrsetref; # shallow RR clone
-		my $RR	  = $RR[0];
-		my $class = $RR->class;
-		my $type  = $RR->type;
+		my @RR	  = map bless( {%$_}, ref($_) ), @$rrsetref;   # shallow RR clone
+		my $rr	  = $RR[0];
+		my $class = $rr->class;
+		my $type  = $rr->type;
+		my $ttl	  = $self->orgttl;
 
-		my $ttl = $self->orgttl;
 		my %table;
 		foreach my $RR (@RR) {
 			my $ident = $RR->{owner}->canonical;
@@ -557,11 +534,11 @@ sub _CreateSig {
 		my $self = shift;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $SEC{$algorithm};
+		my $class     = $DNSSEC_siggen{$algorithm};
 
 		eval {
-			die "algorithm $algorithm not supported" unless $class;
-			$self->sigbin( $class->sign(@_) );
+			return $self->sigbin( $class->sign(@_) ) if $class;
+			die "algorithm $algorithm not supported\n";
 		} || croak "${@}signature generation failed";
 	}
 }
@@ -572,11 +549,11 @@ sub _VerifySig {
 		my $self = shift;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $SEC{$algorithm};
+		my $class     = $DNSSEC_verify{$algorithm};
 
 		my $retval = eval {
-			die "algorithm $algorithm not supported" unless $class;
-			$class->verify( @_, $self->sigbin );
+			return $class->verify( @_, $self->sigbin ) if $class;
+			die "algorithm $algorithm not supported\n";
 		};
 
 		unless ($retval) {
@@ -606,8 +583,8 @@ __END__
 
     use Net::DNS::SEC;
     $sigrr = create Net::DNS::RR::RRSIG( \@rrset, $keypath,
-					sigex => 20171231010101
-					sigin => 20171201010101
+					sigex => 20191231010101
+					sigin => 20191201010101
 					);
 
     $sigrr->verify( \@rrset, $keyrr ) || die $sigrr->vrfyerrstr;
@@ -731,8 +708,8 @@ Create a signature over a RR set.
     $sigrr = create Net::DNS::RR::RRSIG( \@rrsetref, $keypath );
 
     $sigrr = create Net::DNS::RR::RRSIG( \@rrsetref, $keypath,
-					sigex => 20171231010101
-					sigin => 20171201010101
+					sigex => 20191231010101
+					sigin => 20191201010101
 					);
     $sigrr->print;
 
@@ -758,8 +735,8 @@ containing the private key as generated by dnssec-keygen.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigex  => 20171231010101,	# signature expiration
-	sigin  => 20171201010101,	# signature inception
+	sigex  => 20191231010101,	# signature expiration
+	sigin  => 20191201010101,	# signature inception
 	sigval => 30,			# validity window (days)
 	ttl    => 3600			# TTL
 
@@ -809,9 +786,9 @@ a program that comes with the ISC BIND distribution.
     dnssec-keygen -a 14	-f ksk	ecdsa.example.
     dnssec-keygen -a 14		ecdsa.example.
 
-Do not change the name of the file generated by dnssec-keygen.
-The create method uses the filename to determine the keyowner,
-algorithm and the keyid (keytag).
+Do not change the name of the private key file.
+The create method uses the filename as generated by dnssec-keygen
+to determine the keyowner, algorithm, and the keyid (keytag).
 
 
 =head1 REMARKS
@@ -820,17 +797,20 @@ The code is not optimised for speed.
 It is probably not suitable to be used for signing large zones.
 
 If this code is still around in 2100 (not a leap year) you will
-need to check for proper handling of times ...
+need to check for proper handling of times after 28th February.
 
 =head1 ACKNOWLEDGMENTS
 
-Andy Vaskys (Network Associates Laboratories) supplied the code for
-handling RSA with SHA1 (Algorithm 5).
+Although their original code may have disappeared following redesign of
+Net::DNS, Net::DNS::SEC and the OpenSSL API, the following individual
+contributors deserve to be recognised for their significant influence
+on the development of the RRSIG package.
 
-T.J. Mather, the Crypt::OpenSSL::DSA maintainer, for his quick
-responses to bug report and feature requests.
+Andy Vaskys (Network Associates Laboratories) supplied code for RSA.
 
-Dick Franks added support for elliptic curve signatures.
+T.J. Mather provided support for the DSA algorithm.
+
+Dick Franks added support for elliptic curve and Edwards curve algorithms.
 
 Mike McCauley created the Crypt::OpenSSL::ECDSA perl extension module
 specifically for this development.
@@ -873,8 +853,8 @@ DEALINGS IN THE SOFTWARE.
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::SEC>,
 RFC4034, RFC6840, RFC3755,
 L<Net::DNS::SEC::DSA>,
-L<Net::DNS::SEC::ECCGOST>,
 L<Net::DNS::SEC::ECDSA>,
+L<Net::DNS::SEC::EdDSA>,
 L<Net::DNS::SEC::RSA>
 
 L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
