@@ -1,35 +1,17 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1512 2016-10-26 09:14:27Z willem $
+# $Id: Base.pm 1762 2020-02-02 21:39:02Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1512 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1762 $)[1];
 
 
-# Allow taint tests to be optimised away when appropriate.
-use constant UTIL  => defined eval 'require Scalar::Util';
-use constant UNCND => $] < 5.008;	## eval '${^TAINT}' breaks old compilers
-use constant TAINT => UTIL && ( UNCND || eval '${^TAINT}' );
-
-
-use strict;
-use integer;
-use Carp;
-use IO::Select;
-use IO::Socket;
-
-use Net::DNS::RR;
-use Net::DNS::Packet;
-
-use constant INT16SZ  => 2;
-use constant PACKETSZ => 512;
-
 #
 #  Implementation notes wrt IPv6 support when using perl before 5.20.0.
 #
 #  In general we try to be gracious to those stacks that do not have IPv6 support.
-#  We test that by means of the availability of IO::Socket::INET6 or IO::Socket::IP
+#  The socket code is conditionally compiled depending upon the availability of
+#  the IO::Socket::IP package.
 #
 #  We have chosen not to use mapped IPv4 addresses, there seem to be issues
 #  with this; as a result we use separate sockets for each family type.
@@ -41,28 +23,43 @@ use constant PACKETSZ => 512;
 #  are provided to control IPv6 behaviour for test purposes.
 #
 # Olaf Kolkman, RIPE NCC, December 2003.
-# [Revised March 2016]
+# [Revised March 2016, June 2018]
 
 
-use constant USE_SOCKET_IP => defined eval 'use Socket 1.98; use IO::Socket::IP 0.32; 1;';
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1';
 
-use constant USE_SOCKET_INET => defined eval 'require IO::Socket::INET';
-
-use constant USE_SOCKET_INET6 => defined eval 'require IO::Socket::INET6';
-
-use constant IPv4 => USE_SOCKET_IP || USE_SOCKET_INET;
-use constant IPv6 => USE_SOCKET_IP || USE_SOCKET_INET6;
+use constant IPv6 => USE_SOCKET_IP;
 
 
 # If SOCKSified Perl, use TCP instead of UDP and keep the socket open.
 use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 
 
+# Allow taint tests to be optimised away when appropriate.
+use constant UNCND => $] < 5.008;	## eval '${^TAINT}' breaks old compilers
+use constant TAINT => UNCND || eval '${^TAINT}';
+use constant TESTS => TAINT && defined eval 'require Scalar::Util';
+
+
+use strict;
+use warnings;
+use integer;
+use Carp;
+use IO::Select;
+use IO::Socket;
+
+use Net::DNS::RR;
+use Net::DNS::Packet;
+
+use constant PACKETSZ => 512;
+
+
 #
 # Set up a closure to be our class data.
 #
 {
 	my $defaults = bless {
+		nameservers	=> [qw(::1 127.0.0.1)],
 		nameserver4	=> ['127.0.0.1'],
 		nameserver6	=> ['::1'],
 		port		=> 53,
@@ -77,6 +74,7 @@ use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 		recurse		=> 1,
 		defnames	=> 1,
 		dnsrch		=> 1,
+		ndots		=> 1,
 		debug		=> 0,
 		tcp_timeout	=> 120,
 		udp_timeout	=> 30,
@@ -88,7 +86,8 @@ use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 		udppacketsize	=> 0,	# value bounded below by PACKETSZ
 		force_v4	=> ( IPv6 ? 0 : 1 ),
 		force_v6	=> 0,	# only relevant if IPv6 is supported
-		prefer_v4	=> ( IPv6 ? 1 : 0 ),
+		prefer_v4	=> 0,
+		prefer_v6	=> 0,
 		},
 			__PACKAGE__;
 
@@ -97,10 +96,22 @@ use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 }
 
 
+my $warned;
+
+sub _deprecate {
+	carp join ' ', 'deprecated method;', pop(@_) unless $warned++;
+}
+
+
+sub _untaint {
+	return TAINT ? map ref($_) ? [_untaint(@$_)] : do { /^(.*)$/; $1 }, @_ : @_;
+}
+
+
 # These are the attributes that the user may specify in the new() constructor.
 my %public_attr = (
-	map( {$_ => $_} keys %{&_defaults}, qw(domain nameserver nameservers prefer_v6 srcaddr) ),
-	map( {$_ => 0} qw(nameserver4 nameserver6 srcaddr4 srcaddr6) ),
+	map( ( $_ => $_ ), keys %{&_defaults}, qw(domain nameserver srcaddr) ),
+	map( ( $_ => 0 ), qw(nameserver4 nameserver6 srcaddr4 srcaddr6) ),
 	);
 
 
@@ -112,12 +123,11 @@ sub new {
 	my $self;
 	my $base = $class->_defaults;
 	my $init = $initial;
-	$initial ||= bless {%$base}, $class;
+	$initial ||= [%$base];
 	if ( my $file = $args{config_file} ) {
-		$self = bless {%$initial}, $class;
-		$self->_read_config_file($file);		# user specified config
-		$self->nameserver( map { m/^(.*)$/; $1 } $self->nameserver );
-		$self->searchlist( map { m/^(.*)$/; $1 } $self->searchlist );
+		my $conf = bless {@$initial}, $class;
+		$conf->_read_config_file($file);		# user specified config
+		$self = bless {_untaint(%$conf)}, $class;
 		%$base = %$self unless $init;			# define default configuration
 
 	} elsif ($init) {
@@ -146,28 +156,30 @@ my %resolv_conf = (			## map traditional resolv.conf option names
 	timeout	 => 'retrans',
 	);
 
-my %env_option = (			## any resolver attribute except as listed below
+my %res_option = (			## any resolver attribute plus those listed above
 	%public_attr,
 	%resolv_conf,
-	map { $_ => 0 } qw(nameserver nameservers domain searchlist),
 	);
 
+sub _option {
+	my ( $self, $name, @value ) = @_;
+	my $attribute = $res_option{lc $name} || return;
+	push @value, 1 unless scalar @value;
+	$self->$attribute(@value);
+}
+
+
 sub _read_env {				## read resolver config environment variables
 	my $self = shift;
 
-	$self->nameservers( map split, $ENV{RES_NAMESERVERS} ) if exists $ENV{RES_NAMESERVERS};
+	$self->searchlist( map split, $ENV{LOCALDOMAIN} ) if defined $ENV{LOCALDOMAIN};
 
-	$self->domain( $ENV{LOCALDOMAIN} ) if exists $ENV{LOCALDOMAIN};
+	$self->nameservers( map split, $ENV{RES_NAMESERVERS} ) if defined $ENV{RES_NAMESERVERS};
 
-	$self->searchlist( map split, $ENV{RES_SEARCHLIST} ) if exists $ENV{RES_SEARCHLIST};
+	$self->searchlist( map split, $ENV{RES_SEARCHLIST} ) if defined $ENV{RES_SEARCHLIST};
 
-	if ( exists $ENV{RES_OPTIONS} ) {
-		foreach ( map split, $ENV{RES_OPTIONS} ) {
-			my ( $name, $val ) = split( m/:/, $_, 2 );
-			my $attribute = $env_option{$name} || next;
-			$val = 1 unless defined $val;
-			$self->$attribute($val);
-		}
+	foreach ( map split, $ENV{RES_OPTIONS} || '' ) {
+		$self->_option( split m/:/ );
 	}
 }
 
@@ -176,30 +188,19 @@ sub _read_config_file {			## read resolver config file
 	my $self = shift;
 	my $file = shift;
 
-	my @ns;
+	my $filehandle;
+	open( $filehandle, '<', $file ) or croak "$file: $!";
 
-	local *FILE;
-
-	open( FILE, $file ) or croak "Could not open $file: $!";
+	my @nameserver;
+	my @searchlist;
 
 	local $_;
-	while (<FILE>) {
+	while (<$filehandle>) {
 		s/[;#].*$//;					# strip comments
 
 		/^nameserver/ && do {
 			my ( $keyword, @ip ) = grep defined, split;
-			push @ns, @ip;
-			next;
-		};
-
-		/^option/ && do {
-			my ( $keyword, @option ) = grep defined, split;
-			foreach (@option) {
-				my ( $name, $val ) = split( m/:/, $_, 2 );
-				my $attribute = $resolv_conf{$name} || next;
-				$val = 1 unless defined $val;
-				$self->$attribute($val);
-			}
+			push @nameserver, @ip;
 			next;
 		};
 
@@ -210,15 +211,23 @@ sub _read_config_file {			## read resolver config file
 		};
 
 		/^search/ && do {
-			my ( $keyword, @searchlist ) = grep defined, split;
-			$self->searchlist(@searchlist);
+			my ( $keyword, @domain ) = grep defined, split;
+			push @searchlist, @domain;
 			next;
 		};
+
+		/^option/ && do {
+			my ( $keyword, @option ) = grep defined, split;
+			foreach (@option) {
+				$self->_option( split m/:/ );
+			}
+		};
 	}
 
-	close(FILE);
+	close($filehandle);
 
-	$self->nameservers(@ns);
+	$self->nameservers(@nameserver) if @nameserver;
+	$self->searchlist(@searchlist)	if @searchlist;
 }
 
 
@@ -227,22 +236,21 @@ sub string {
 	$self = $self->_defaults unless ref($self);
 
 	my @nslist = $self->nameservers();
-	my $domain = $self->domain;
+	my ($force)  = ( grep( $self->{$_}, qw(force_v6 force_v4) ),   'force_v4' );
+	my ($prefer) = ( grep( $self->{$_}, qw(prefer_v6 prefer_v4) ), 'prefer_v4' );
 	return <<END;
 ;; RESOLVER state:
-;; domain	= $domain
-;; searchlist	= @{$self->{searchlist}}
 ;; nameservers	= @nslist
+;; searchlist	= @{$self->{searchlist}}
 ;; defnames	= $self->{defnames}	dnsrch		= $self->{dnsrch}
+;; igntc	= $self->{igntc}	usevc		= $self->{usevc}
+;; recurse	= $self->{recurse}	port		= $self->{port}
 ;; retrans	= $self->{retrans}	retry		= $self->{retry}
-;; recurse	= $self->{recurse}	igntc		= $self->{igntc}
-;; usevc	= $self->{usevc}	port		= $self->{port}
 ;; tcp_timeout	= $self->{tcp_timeout}	persistent_tcp	= $self->{persistent_tcp}
 ;; udp_timeout	= $self->{udp_timeout}	persistent_udp	= $self->{persistent_udp}
-;; prefer_v4	= $self->{prefer_v4}	force_v4	= $self->{force_v4}
-;; debug	= $self->{debug}	force_v6	= $self->{force_v6}
+;; ${prefer}	= $self->{$prefer}	${force}	= $self->{$force}
+;; debug	= $self->{debug}	ndots		= $self->{ndots}
 END
-
 }
 
 
@@ -270,57 +278,57 @@ sub nameservers {
 	my $self = shift;
 	$self = $self->_defaults unless ref($self);
 
-	my ( @ipv4, @ipv6 );
+	my @ip;
 	foreach my $ns ( grep defined, @_ ) {
-		do { push @ipv6, $ns; next } if _ipv6($ns);
-		do { push @ipv4, $ns; next } if _ipv4($ns);
+		if ( _ipv4($ns) || _ipv6($ns) ) {
+			push @ip, $ns;
 
-		my $defres = ref($self)->new( debug => $self->{debug} );
-		$defres->{persistent} = $self->{persistent};
+		} else {
+			my $defres = ref($self)->new( debug => $self->{debug} );
+			$defres->{persistent} = $self->{persistent};
 
-		my $names  = {};
-		my $packet = $defres->search( $ns, 'A' );
-		my @iplist = _cname_addr( $packet, $names );
+			my $names  = {};
+			my $packet = $defres->send( $ns, 'A' );
+			my @iplist = _cname_addr( $packet, $names );
 
-		if (IPv6) {
-			$packet = $defres->search( $ns, 'AAAA' );
-			push @iplist, _cname_addr( $packet, $names );
+			if (IPv6) {
+				$packet = $defres->send( $ns, 'AAAA' );
+				push @iplist, _cname_addr( $packet, $names );
+			}
+
+			my %unique = map( ( $_ => $_ ), @iplist );
+
+			my @address = values(%unique);		# tainted
+			carp "unresolvable name: $ns" unless scalar @address;
+
+			push @ip, @address;
 		}
-
-		$self->errorstring( $defres->errorstring );
-
-		my %address = map { ( $_ => $_ ) } @iplist;	# tainted
-		my @unique = values %address;
-		carp "unresolvable name: $ns" unless @unique;
-		push @ipv4, grep _ipv4($_), @unique;
-		push @ipv6, grep _ipv6($_), @unique;
 	}
 
-	unless ( defined wantarray ) {
-		$self->{nameserver4} = \@ipv4;
-		$self->{nameserver6} = \@ipv6;
-		return;
-	}
-
-	if ( scalar @_ ) {
+	if ( scalar(@_) || !defined(wantarray) ) {
+		my @ipv4 = grep _ipv4($_), @ip;
+		my @ipv6 = grep _ipv6($_), @ip;
+		$self->{nameservers} = \@ip;
 		$self->{nameserver4} = \@ipv4;
 		$self->{nameserver6} = \@ipv6;
 	}
 
-	my @ns4 = $self->force_v6 ? () : @{$self->{nameserver4}};
-	my @ns6 = $self->force_v4 ? () : @{$self->{nameserver6}};
-	my @returnval = $self->{prefer_v4} ? ( @ns4, @ns6 ) : ( @ns6, @ns4 );
+	my @ns4 = $self->{force_v6} ? () : @{$self->{nameserver4}};
+	my @ns6 = $self->{force_v4} ? () : @{$self->{nameserver6}};
+	my @nameservers = @{$self->{nameservers}};
+	@nameservers = ( @ns4, @ns6 ) if $self->{prefer_v4} || !scalar(@ns6);
+	@nameservers = ( @ns6, @ns4 ) if $self->{prefer_v6} || !scalar(@ns4);
 
-	return @returnval if scalar @returnval;
+	return @nameservers if scalar @nameservers;
 
 	my $error = 'no nameservers';
 	$error = 'IPv4 transport disabled' if scalar(@ns4) < scalar @{$self->{nameserver4}};
 	$error = 'IPv6 transport disabled' if scalar(@ns6) < scalar @{$self->{nameserver6}};
 	$self->errorstring($error);
-	return @returnval;
+	return @nameservers;
 }
 
-sub nameserver { &nameservers; }				# uncoverable pod
+sub nameserver { &nameservers; }
 
 sub _cname_addr {
 
@@ -339,20 +347,22 @@ sub _cname_addr {
 }
 
 
-sub answerfrom {
-	my $self = shift;
-	$self->{answerfrom} = shift if scalar @_;
-	return $self->{answerfrom};
+sub replyfrom {
+	return shift->{replyfrom};
 }
 
+sub answerfrom { &replyfrom; }					# uncoverable pod
+
+
 sub _reset_errorstring {
 	shift->{errorstring} = '';
 }
 
 sub errorstring {
 	my $self = shift;
-	$self->{errorstring} = shift if scalar @_;
-	return $self->{errorstring};
+	my $text = shift || return $self->{errorstring};
+	$self->_diag( 'errorstring:', $text );
+	return $self->{errorstring} = $text;
 }
 
 
@@ -360,14 +370,11 @@ sub query {
 	my $self = shift;
 	my $name = shift || '.';
 
-	# resolve name containing no dots or colons by appending domain
-	my @sfix = $self->{defnames} && $name !~ m/[:.]/ ? $self->domain : ();
+	my @sfix = $self->{defnames} && ( $name !~ m/[.:]/ ) ? $self->domain : ();
+
 	my $fqdn = join '.', $name, @sfix;
-
 	$self->_diag( 'query(', $fqdn, @_, ')' );
-
 	my $packet = $self->send( $fqdn, @_ ) || return;
-
 	return $packet->header->ancount ? $packet : undef;
 }
 
@@ -378,17 +385,16 @@ sub search {
 	return $self->query(@_) unless $self->{dnsrch};
 
 	my $name = shift || '.';
-	my @sfix = $self->searchlist;
-	my @list = $name =~ m/[.]/ ? ( undef, @sfix ) : ( @sfix, undef );
+	my $dots = $name =~ tr/././;
 
-	foreach my $suffix ( $name =~ m/:|\.\d*$/ ? undef : @list ) {
+	my @sfix = ( $dots < $self->{ndots} ) ? @{$self->{searchlist}} : ();
+	my ( $one, @more ) = ( $name =~ m/:|\.\d*$/ ) ? () : ( $dots ? ( undef, @sfix ) : @sfix );
+
+	foreach my $suffix ( $one, @more ) {
 		my $fqname = $suffix ? join( '.', $name, $suffix ) : $name;
-
 		$self->_diag( 'search(', $fqname, @_, ')' );
-
 		my $packet = $self->send( $fqname, @_ ) || next;
-
-		return $packet->header->ancount ? $packet : next;
+		return $packet if $packet->header->ancount;
 	}
 
 	return undef;
@@ -400,13 +406,15 @@ sub send {
 	my $packet	= $self->_make_query_packet(@_);
 	my $packet_data = $packet->data;
 
+	$self->_reset_errorstring;
+
 	return $self->_send_tcp( $packet, $packet_data )
 			if $self->{usevc} || length $packet_data > $self->_packetsz;
 
-	my $ans = $self->_send_udp( $packet, $packet_data ) || return;
+	my $reply = $self->_send_udp( $packet, $packet_data ) || return;
 
-	return $ans if $self->{igntc};
-	return $ans unless $ans->header->tc;
+	return $reply if $self->{igntc};
+	return $reply unless $reply->header->tc;
 
 	$self->_diag('packet truncated: retrying using TCP');
 	$self->_send_tcp( $packet, $packet_data );
@@ -414,13 +422,12 @@ sub send {
 
 
 sub _send_tcp {
-	my ( $self, $packet, $packet_data ) = @_;
+	my ( $self, $query, $query_data ) = @_;
 
-	$self->_reset_errorstring;
-
-	my $tcp_packet = pack 'n a*', length($packet_data), $packet_data;
+	my $tcp_packet = pack 'n a*', length($query_data), $query_data;
 	my @ns = $self->nameservers();
-	my $lastanswer;
+	my $fallback;
+	my $timeout = $self->{tcp_timeout};
 
 	foreach my $ip (@ns) {
 		my $socket = $self->_create_tcp_socket($ip) || next;
@@ -431,63 +438,57 @@ sub _send_tcp {
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
 
-		next unless $select->can_read( $self->{tcp_timeout} );
+		next unless $select->can_read($timeout);	# uncoverable branch true
 
 		my $buffer = _read_tcp($socket);
-		$self->answerfrom($ip);
-		$self->_diag( 'answer from', "[$ip]", length($buffer), 'bytes' );
+		$self->{replyfrom} = $ip;
+		$self->_diag( 'reply from', "[$ip]", length($buffer), 'bytes' );
 
-		my $ans = Net::DNS::Packet->new( \$buffer, $self->{debug} );
+		my $reply = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 		$self->errorstring($@);
+		next unless $self->_accept_reply( $reply, $query );
+		$reply->from($ip);
 
-		next unless $ans;
+		if ( $self->{tsig_rr} && !$reply->verify($query) ) {
+			$self->errorstring( $reply->verifyerr );
+			next;
+		}
 
-		$ans->answerfrom($ip);
-		$lastanswer = $ans;
-
-		my $rcode = $ans->header->rcode;
-		last if $rcode eq 'NOERROR';
-		last if $rcode eq 'NXDOMAIN';
-		$self->errorstring($rcode);
+		my $rcode = $reply->header->rcode;
+		return $reply if $rcode eq 'NOERROR';
+		return $reply if $rcode eq 'NXDOMAIN';
+		$fallback = $reply;
 	}
 
-	return unless $lastanswer;
-
-	if ( $self->{tsig_rr} && !$lastanswer->verify($packet) ) {
-		$self->_diag( $self->errorstring( $lastanswer->verifyerr ) );
-		return;
-	}
-
-	$self->errorstring( $lastanswer->header->rcode );	# historical quirk
-	return $lastanswer;
+	$self->{errorstring} = $fallback->header->rcode if $fallback;
+	$self->errorstring('query timed out') unless $self->{errorstring};
+	return $fallback;
 }
 
 
 sub _send_udp {
-	my ( $self, $packet, $packet_data ) = @_;
-
-	$self->_reset_errorstring;
+	my ( $self, $query, $query_data ) = @_;
 
 	my @ns	    = $self->nameservers;
 	my $port    = $self->{port};
-	my $queryid = $packet->header->id;
 	my $retrans = $self->{retrans} || 1;
 	my $retry   = $self->{retry} || 1;
-	my $select  = IO::Select->new();
 	my $servers = scalar(@ns);
 	my $timeout = $servers ? do { no integer; $retrans / $servers } : 0;
-	my $lastanswer;
+	my $fallback;
 
 	# Perform each round of retries.
 RETRY: for ( 1 .. $retry ) {					# assumed to be a small number
 
 		# Try each nameserver.
+		my $select = IO::Select->new();
+
 NAMESERVER: foreach my $ns (@ns) {
 
-			# Construct an array of 3 element arrays
+			# state vector replaces corresponding element of @ns array
 			unless ( ref $ns ) {
-				my $socket = $self->_create_udp_socket($ns) || next;
 				my $dst_sockaddr = $self->_create_dst_sockaddr( $ns, $port );
+				my $socket = $self->_create_udp_socket($ns) || next;
 				$ns = [$socket, $ns, $dst_sockaddr];
 			}
 
@@ -496,60 +497,51 @@ NAMESERVER: foreach my $ns (@ns) {
 
 			$self->_diag( 'udp send', "[$ip]:$port" );
 
-			$socket->send( $packet_data, 0, $dst_sockaddr );
+			$select->add($socket);
+			$socket->send( $query_data, 0, $dst_sockaddr );
 			$self->errorstring( $$ns[3] = $! );
 
 			# handle failure to detect taint inside socket->send()
 			die 'Insecure dependency while running with -T switch'
-					if TAINT && Scalar::Util::tainted($dst_sockaddr);
-
-			$select->add($socket);
+					if TESTS && Scalar::Util::tainted($dst_sockaddr);
 
+			my $reply;
 			while ( my ($socket) = $select->can_read($timeout) ) {
-				$select->remove($socket);
-
 				my $peer = $socket->peerhost;
-				$self->answerfrom($peer);
+				$self->{replyfrom} = $peer;
 
 				my $buffer = _read_udp( $socket, $self->_packetsz );
-				$self->_diag( "answer from [$peer]", length($buffer), 'bytes' );
+				$self->_diag( "reply from [$peer]", length($buffer), 'bytes' );
 
-				my $ans = Net::DNS::Packet->new( \$buffer, $self->{debug} );
+				my $packet = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 				$self->errorstring($@);
+				next unless $self->_accept_reply( $packet, $query );
+				$reply = $packet;
+				$reply->from($peer);
+				last;
+			}					#SELECT LOOP
 
-				next unless $ans;
+			next unless $reply;
 
-				my $header = $ans->header;
-				next unless $header->qr;
-				next unless $header->id == $queryid;
-
-				$ans->answerfrom($peer);
-				$lastanswer = $ans;
-
-				my $rcode = $header->rcode;
-				last if $rcode eq 'NOERROR';
-				last if $rcode eq 'NXDOMAIN';
-
-				$self->errorstring( $$ns[3] = $rcode );
-			}					#SELECTOR LOOP
-
-			next unless $lastanswer;
-
-			if ( $self->{tsig_rr} && !$lastanswer->verify($packet) ) {
-				my $error = $$ns[3] = $lastanswer->verifyerr;
-				$self->_diag( $self->errorstring($error) );
+			if ( $self->{tsig_rr} && !$reply->verify($query) ) {
+				$self->errorstring( $$ns[3] = $reply->verifyerr );
 				next;
 			}
 
-			$self->errorstring( $lastanswer->header->rcode );    # historical quirk
-			return $lastanswer;
+			my $rcode = $reply->header->rcode;
+			return $reply if $rcode eq 'NOERROR';
+			return $reply if $rcode eq 'NXDOMAIN';
+			$fallback = $reply;
+			$$ns[3] = $rcode;
 		}						#NAMESERVER LOOP
+
 		no integer;
 		$timeout += $timeout;
 	}							#RETRY LOOP
 
-	$self->_diag( $self->errorstring('query timed out') ) unless $lastanswer;
-	return;
+	$self->{errorstring} = $fallback->header->rcode if $fallback;
+	$self->errorstring('query timed out') unless $self->{errorstring};
+	return $fallback;
 }
 
 
@@ -558,6 +550,8 @@ sub bgsend {
 	my $packet	= $self->_make_query_packet(@_);
 	my $packet_data = $packet->data;
 
+	$self->_reset_errorstring;
+
 	return $self->_bgsend_tcp( $packet, $packet_data )
 			if $self->{usevc} || length $packet_data > $self->_packetsz;
 
@@ -568,8 +562,6 @@ sub bgsend {
 sub _bgsend_tcp {
 	my ( $self, $packet, $packet_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $tcp_packet = pack 'n a*', length($packet_data), $packet_data;
 
 	foreach my $ip ( $self->nameservers ) {
@@ -577,15 +569,16 @@ sub _bgsend_tcp {
 
 		$self->_diag( 'bgsend', "[$ip]" );
 
+		$socket->blocking(0);
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
+		$socket->blocking(1);
 
 		my $expire = time() + $self->{tcp_timeout};
 		${*$socket}{net_dns_bg} = [$expire, $packet];
 		return $socket;
 	}
 
-	$self->_diag( $self->errorstring );
 	return undef;
 }
 
@@ -593,13 +586,11 @@ sub _bgsend_tcp {
 sub _bgsend_udp {
 	my ( $self, $packet, $packet_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $port = $self->{port};
 
 	foreach my $ip ( $self->nameservers ) {
-		my $socket = $self->_create_udp_socket($ip) || next;
 		my $dst_sockaddr = $self->_create_dst_sockaddr( $ip, $port );
+		my $socket = $self->_create_udp_socket($ip) || next;
 
 		$self->_diag( 'bgsend', "[$ip]:$port" );
 
@@ -608,14 +599,13 @@ sub _bgsend_udp {
 
 		# handle failure to detect taint inside $socket->send()
 		die 'Insecure dependency while running with -T switch'
-				if TAINT && Scalar::Util::tainted($dst_sockaddr);
+				if TESTS && Scalar::Util::tainted($dst_sockaddr);
 
 		my $expire = time() + $self->{udp_timeout};
 		${*$socket}{net_dns_bg} = [$expire, $packet];
 		return $socket;
 	}
 
-	$self->_diag( $self->errorstring );
 	return undef;
 }
 
@@ -628,13 +618,11 @@ sub bgbusy {
 	my ( $expire, $query, $read ) = @$appendix;
 	return if ref($read);
 
-	unless ( IO::Select->new($handle)->can_read(0.2) ) {
-		return time() <= $expire;
-	}
+	return time() <= $expire unless IO::Select->new($handle)->can_read(0);
 
 	return if $self->{igntc};
-	return unless $query;
 	return unless $handle->socktype() == SOCK_DGRAM;
+	return unless $query;					# SpamAssassin 3.4.1 workaround
 
 	my $ans = $self->_bgread($handle);
 	$$appendix[2] = [$ans];
@@ -642,18 +630,21 @@ sub bgbusy {
 	return unless $ans->header->tc;
 
 	$self->_diag('packet truncated: retrying using TCP');
-	my $tcp = $self->_bgsend_tcp( $query, $query->data );
-	return defined( $_[1] = $tcp ) if $tcp;
+	my $tcp = $self->_bgsend_tcp( $query, $query->data ) || return;
+	return defined( $_[1] = $tcp );
 }
 
 
 sub bgisready {				## historical
-	!&bgbusy;						# uncoverable pod
+	_deprecate('prefer  ! bgbusy(...)');			# uncoverable pod
+	!&bgbusy;
 }
 
 
 sub bgread {
-	while (&bgbusy) { next; }				# side effect: TCP retry
+	while (&bgbusy) {					# side effect: TCP retry
+		IO::Select->new( $_[1] )->can_read(0.02);	# reduce my CPU usage by 3 orders of magnitude
+	}
 	&_bgread;
 }
 
@@ -666,29 +657,40 @@ sub _bgread {
 	my ( $expire, $query, $read ) = @$appendix;
 	return shift(@$read) if ref($read);
 
-	return unless IO::Select->new($handle)->can_read(0);
-
-	my $peer = $handle->peerhost;
-	$self->answerfrom($peer);
-
-	my $dgram = $handle->socktype() == SOCK_DGRAM;
-	my $buffer = $dgram ? _read_udp( $handle, $self->_packetsz ) : _read_tcp($handle);
-	$self->_diag( "answer from [$peer]", length($buffer), 'bytes' );
-
-	my $ans = Net::DNS::Packet->new( \$buffer, $self->{debug} );
-	$self->errorstring($@);
-	return unless $ans;
-
-	if ( $self->{tsig_rr} && !$ans->verify($query) ) {
-		$self->errorstring( $ans->verifyerr );
+	my $select = IO::Select->new($handle);
+	unless ( $select->can_read(0) ) {
+		$self->errorstring('timed out');
 		return;
 	}
 
-	my $header = $ans->header;
+	my $peer = $self->{replyfrom} = $handle->peerhost;
+
+	my $dgram = $handle->socktype() == SOCK_DGRAM;
+	my $buffer = $dgram ? _read_udp( $handle, $self->_packetsz ) : _read_tcp($handle);
+	$self->_diag( "reply from [$peer]", length($buffer), 'bytes' );
+
+	my $reply = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
+	$self->errorstring($@);
+	return unless $self->_accept_reply( $reply, $query );
+	$reply->from($peer);
+
+	return $reply unless $self->{tsig_rr} && !$reply->verify($query);
+	$self->errorstring( $reply->verifyerr );
+	return;
+}
+
+
+sub _accept_reply {
+	my ( $self, $reply, $query ) = @_;
+
+	return unless $reply;
+
+	my $header = $reply->header;
 	return unless $header->qr;
 
-	$ans->answerfrom($peer);
-	return ( ref($query) && ( $header->id != $query->header->id ) ) ? undef : $ans;
+	return if $query && $header->id != $query->header->id;
+
+	$self->errorstring( $header->rcode );			# historical quirk
 }
 
 
@@ -696,29 +698,33 @@ sub axfr {				## zone transfer
 	eval {
 		my $self = shift;
 
+		# initialise iterator state vector
 		my ( $select, $verify, @rr, $soa ) = $self->_axfr_start(@_);
 
 		my $iterator = sub {	## iterate over RRs
 			my $rr = shift(@rr);
 
 			if ( ref($rr) eq 'Net::DNS::RR::SOA' ) {
-				return $soa = $rr unless $soa;
-				$select = undef;
-				return if $rr->encode eq $soa->encode;
-				croak $self->errorstring('mismatched final SOA');
+				if ($soa) {
+					$select = undef;
+					return if $rr->encode eq $soa->encode;
+					croak $self->errorstring('mismatched final SOA');
+				}
+				$soa = $rr;
 			}
 
-			return $rr if scalar @rr;
+			unless ( scalar @rr ) {
+				my $reply;			# refill @rr
+				( $reply, $verify ) = $self->_axfr_next( $select, $verify );
+				@rr = $reply->answer;
+			}
 
-			my $reply;
-			( $reply, $verify ) = $self->_axfr_next( $select, $verify );
-			@rr = $reply->answer;
 			return $rr;
 		};
 
 		return $iterator unless wantarray;
 
-		my @zone;					# assemble whole zone
+		my @zone;		## subvert iterator to assemble entire zone
 		while ( my $rr = $iterator->() ) {
 			push @zone, $rr, @rr;			# copy RRs en bloc
 			@rr = pop(@zone);			# leave last one in @rr
@@ -729,13 +735,15 @@ sub axfr {				## zone transfer
 
 
 sub axfr_start {			## historical
-	my $self = shift;					# uncoverable pod
+	_deprecate('prefer  $iterator = $self->axfr(...)');	# uncoverable pod
+	my $self = shift;
 	defined( $self->{axfr_iter} = $self->axfr(@_) );
 }
 
 
 sub axfr_next {				## historical
-	shift->{axfr_iter}->();					# uncoverable pod
+	_deprecate('prefer  $iterator->()');			# uncoverable pod
+	shift->{axfr_iter}->();
 }
 
 
@@ -789,7 +797,7 @@ sub _axfr_next {
 	my ($socket) = $select->can_read( $self->{tcp_timeout} );
 	croak $self->errorstring('timed out') unless $socket;
 
-	$self->answerfrom( $socket->peerhost );
+	$self->{replyfrom} = $socket->peerhost;
 
 	my $buffer = _read_tcp($socket);
 	$self->_diag( 'received', length($buffer), 'bytes' );
@@ -811,25 +819,17 @@ sub _axfr_next {
 sub _read_tcp {
 	my $socket = shift;
 
-	my $size_buf = '';
-	$socket->recv( $size_buf, INT16SZ );
-	my ($unread) = unpack 'n*', $size_buf;
+	my ( $buffer, $s1, $s2 );
+	$socket->recv( $s1, 2 );				# one lump
+	$socket->recv( $s2, 2 - length $s1 );			# or two?
+	my $size = unpack 'n', pack( 'a*a*@2', $s1, $s2 );
 
-	my $buffer = '';
-	while ($unread) {
+	$socket->recv( $buffer, $size );			# initial read
 
-		# During some of my tests recv() returned undef even
-		# though there was no error.  Checking for the amount
-		# of data read appears to work around that problem.
-
-		my $read_buf = '';
-		$socket->recv( $read_buf, $unread );
-
-		my $read = length $read_buf;
-		last unless $read;				# uncoverable branch
-
-		$buffer .= $read_buf;
-		$unread -= $read;
+	while ( length($buffer) < $size ) {
+		my $fragment;
+		$socket->recv( $fragment, $size - length($buffer) );
+		$buffer .= $fragment || last;
 	}
 	return $buffer;
 }
@@ -859,44 +859,31 @@ sub _create_tcp_socket {
 		$self->_diag('socket disconnected (trying to connect)');
 	}
 
-	my $dstport = $self->{port};
-	my $srcport = $self->{srcport};
-	my $timeout = $self->{tcp_timeout} || 1;
+	my $ip6_addr = IPv6 && _ipv6($ip);
 
-	if (USE_SOCKET_IP) {
-		my $srcaddr = _ipv6($ip) ? $self->{srcaddr6} : $self->{srcaddr4};
-		$socket = IO::Socket::IP->new(
-			LocalAddr => $srcaddr,
-			LocalPort => $srcport,
-			PeerAddr  => $ip,
-			PeerPort  => $dstport,
-			Proto	  => 'tcp',
-			Timeout	  => $timeout
-			)
+	$socket = IO::Socket::IP->new(
+		LocalAddr => $ip6_addr ? $self->{srcaddr6} : $self->{srcaddr4},
+		LocalPort => $self->{srcport},
+		PeerAddr  => $ip,
+		PeerPort  => $self->{port},
+		Proto	  => 'tcp',
+		Timeout	  => $self->{tcp_timeout},
+		)
+			if USE_SOCKET_IP;
 
-	} elsif ( not _ipv6($ip) ) {
+	unless (USE_SOCKET_IP) {
 		$socket = IO::Socket::INET->new(
 			LocalAddr => $self->{srcaddr4},
-			LocalPort => ( $srcport || undef ),
+			LocalPort => $self->{srcport} || undef,
 			PeerAddr  => $ip,
-			PeerPort  => $dstport,
+			PeerPort  => $self->{port},
 			Proto	  => 'tcp',
-			Timeout	  => $timeout
+			Timeout	  => $self->{tcp_timeout},
 			)
-
-	} elsif (USE_SOCKET_INET6) {
-		$socket = IO::Socket::INET6->new(
-			LocalAddr => $self->{srcaddr6},
-			LocalPort => ( $srcport || undef ),
-			PeerAddr  => $ip,
-			PeerPort  => $dstport,
-			Proto	  => 'tcp',
-			Timeout	  => $timeout
-			);
+				unless $ip6_addr;
 	}
 
 	$self->{persistent}{$sock_key} = $self->{persistent_tcp} ? $socket : undef;
-	$self->errorstring("no socket $sock_key $!") unless $socket;
 	return $socket;
 }
 
@@ -910,64 +897,49 @@ sub _create_udp_socket {
 	my $socket;
 	return $socket if $socket = $self->{persistent}{$sock_key};
 
-	my $srcport = $self->{srcport};
+	$socket = IO::Socket::IP->new(
+		LocalAddr => $ip6_addr ? $self->{srcaddr6} : $self->{srcaddr4},
+		LocalPort => $self->{srcport},
+		Proto	  => 'udp',
+		Type	  => SOCK_DGRAM
+		)
+			if USE_SOCKET_IP;
 
-	if (USE_SOCKET_IP) {
-		my $srcaddr = $ip6_addr ? $self->{srcaddr6} : $self->{srcaddr4};
-		$socket = IO::Socket::IP->new(
-			LocalAddr => $srcaddr,
-			LocalPort => $srcport,
-			Proto	  => 'udp',
-			Type	  => SOCK_DGRAM
-			)
-
-	} elsif ( not $ip6_addr ) {
+	unless (USE_SOCKET_IP) {
 		$socket = IO::Socket::INET->new(
 			LocalAddr => $self->{srcaddr4},
-			LocalPort => ( $srcport || undef ),
+			LocalPort => $self->{srcport} || undef,
 			Proto	  => 'udp',
 			Type	  => SOCK_DGRAM
 			)
-
-	} elsif (USE_SOCKET_INET6) {
-		$socket = IO::Socket::INET6->new(
-			LocalAddr => $self->{srcaddr6},
-			LocalPort => ( $srcport || undef ),
-			Proto	  => 'udp',
-			Type	  => SOCK_DGRAM
-			);
+				unless $ip6_addr;
 	}
 
 	$self->{persistent}{$sock_key} = $self->{persistent_udp} ? $socket : undef;
-	$self->errorstring("no socket $sock_key $!") unless $socket;
 	return $socket;
 }
 
 
-sub _create_dst_sockaddr {		## create UDP destination sockaddr structure
-	my ( $self, $ip, $port ) = @_;
+{
+	no strict qw(subs);
+	my @udp = (
+		flags	 => Socket::AI_NUMERICHOST,
+		protocol => Socket::IPPROTO_UDP,
+		socktype => SOCK_DGRAM
+		);
 
-	no strict;
-	if (USE_SOCKET_IP) {
-		my $family = _ipv6($ip) ? AF_INET6 : AF_INET;
-		my ( $error, $result ) = Socket::getaddrinfo(
-			$ip, $port,
-			{	family	 => $family,
-				flags	 => Socket::AI_NUMERICHOST,
-				protocol => Socket::IPPROTO_UDP,
-				socktype => SOCK_DGRAM
-				} );
-		return $result->{addr};				# NB: error flagged by socket->send
+	my $ip4 = USE_SOCKET_IP ? {family => AF_INET,  @udp} : {};
+	my $ip6 = USE_SOCKET_IP ? {family => AF_INET6, @udp} : {};
 
-	} elsif (USE_SOCKET_INET6) {
-		return sockaddr_in( $port, inet_aton($ip) ) unless _ipv6($ip);
+	sub _create_dst_sockaddr {	## create UDP destination sockaddr structure
+		my ( $self, $ip, $port ) = @_;
 
-		local $^W = 0;					# circumvent perl -w warnings
-		my @res = Socket6::getaddrinfo( $ip, $port, AF_INET6, SOCK_DGRAM, 0, AI_NUMERICHOST );
-		return $res[3];					# NB: error flagged by socket->send
+		unless (USE_SOCKET_IP) {
+			return sockaddr_in( $port, inet_aton($ip) ) unless _ipv6($ip);
+		}
 
-	} else {
-		return sockaddr_in( $port, inet_aton($ip) ) unless _ipv6($ip);
+		( grep ref, Socket::getaddrinfo( $ip, $port, _ipv6($ip) ? $ip6 : $ip4 ), {} )[0]->{addr}
+				if USE_SOCKET_IP;		# NB: errors raised in socket->send
 	}
 }
 
@@ -976,15 +948,17 @@ sub _create_dst_sockaddr {		## create UDP destination sockaddr structure
 
 sub _ipv4 {
 	for (shift) {
-		return /^[0-9.]+\.[0-9]+$/;			# dotted digits
+		return if m/[^.0-9]/;				# dots and digits only
+		return m/\.\d+\./;				# dots separated by digits
 	}
 }
 
 sub _ipv6 {
 	for (shift) {
-		return 1 if /^[:0-9a-f]+:[0-9a-f]*$/i;		# mixed : and hexdigits
-		return 1 if /^[:0-9a-f]+:[0-9.]+$/i;		# prefix + dotted digits
-		return /^[:0-9a-f]+:[0-9a-f]*[%].+$/i;		# RFC4007 scoped address
+		return	 unless m/:.*:/;			# must contain two colons
+		return 1 unless m/[^:0-9A-Fa-f]/;		# colons and hexdigits only
+		return 1 if m/^[:.0-9A-Fa-f]+\%.+$/;		# RFC4007 scoped address
+		return m/^[:0-9A-Fa-f]+:[.0-9]+$/;		# prefix : dotted digits
 	}
 }
 
@@ -1003,7 +977,7 @@ sub _make_query_packet {
 		my $header = $packet->header;
 		$header->ad( $self->{adflag} );			# RFC6840, 5.7
 		$header->cd( $self->{cdflag} );			# RFC6840, 5.9
-		$header->do(1) if $self->dnssec;
+		$header->do(1) if $self->{dnssec};
 		$header->rd( $self->{recurse} );
 	}
 
@@ -1031,30 +1005,26 @@ sub dnssec {
 
 sub force_v6 {
 	my $self = shift;
-	return $self->{force_v6} unless scalar @_;
-	my $value = shift;
-	$self->force_v4(0) if $value;
-	$self->{force_v6} = $value ? 1 : 0;
+	my $value = scalar(@_) ? shift() : $self->{force_v6};
+	$self->{force_v6} = $value ? do { $self->{force_v4} = 0; 1 } : 0;
 }
 
 sub force_v4 {
 	my $self = shift;
-	return $self->{force_v4} unless scalar @_;
-	my $value = shift;
-	$self->force_v6(0) if $value;
-	$self->{force_v4} = $value ? 1 : 0;
+	my $value = scalar(@_) ? shift() : $self->{force_v4};
+	$self->{force_v4} = $value ? do { $self->{force_v6} = 0; 1 } : 0;
 }
 
 sub prefer_v6 {
 	my $self = shift;
-	$self->{prefer_v4} = shift() ? 0 : 1 if scalar @_;
-	$self->{prefer_v4} ? 0 : 1;
+	my $value = scalar(@_) ? shift() : $self->{prefer_v6};
+	$self->{prefer_v6} = $value ? do { $self->{prefer_v4} = 0; 1 } : 0;
 }
 
 sub prefer_v4 {
 	my $self = shift;
-	return $self->{prefer_v4} unless scalar @_;
-	$self->{prefer_v4} = shift() ? 1 : 0;
+	my $value = scalar(@_) ? shift() : $self->{prefer_v4};
+	$self->{prefer_v4} = $value ? do { $self->{prefer_v6} = 0; 1 } : 0;
 }
 
 
@@ -1097,13 +1067,8 @@ sub udppacketsize {
 #
 # Keep this method around. Folk depend on it although it is neither documented nor exported.
 #
-my $warned;
-
 sub make_query_packet {			## historical
-	unless ( $warned++ ) {					# uncoverable pod
-		local $SIG{__WARN__};
-		carp 'deprecated method; see RT#37104';
-	}
+	_deprecate('see RT#37104');				# uncoverable pod
 	&_make_query_packet;
 }
 
@@ -1114,7 +1079,34 @@ sub _diag {				## debug output
 }
 
 
-use vars qw($AUTOLOAD);
+{
+	my $parse_dig = sub {
+		require Net::DNS::ZoneFile;
+
+		my $dug = new Net::DNS::ZoneFile( \*DATA );
+		my @rr	= $dug->read;
+
+		my @auth = grep $_->type eq 'NS', @rr;
+		my %auth = map { lc $_->nsdname => 1 } @auth;
+		my %glue;
+		my @glue = grep $auth{lc $_->name}, @rr;
+		foreach ( grep $_->can('address'), @glue ) {
+			push @{$glue{lc $_->name}}, $_->address;
+		}
+		map @$_, values %glue;
+	};
+
+	my @ip;
+
+	sub _hints {			## default hints
+		@ip = &$parse_dig unless scalar @ip;		# once only, on demand
+		splice @ip, 0, 0, splice( @ip, int( rand scalar @ip ) );    # cut deck
+		return @ip;
+	}
+}
+
+
+our $AUTOLOAD;
 
 sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
@@ -1123,13 +1115,13 @@ sub AUTOLOAD {				## Default method
 
 	my $name = $AUTOLOAD;
 	$name =~ s/.*://;
-	croak "$name: no such method" unless $public_attr{$name};
+	croak qq[unknown method "$name"] unless $public_attr{$name};
 
 	no strict q/refs/;
 	*{$AUTOLOAD} = sub {
 		my $self = shift;
 		$self = $self->_defaults unless ref($self);
-		$self->{$name} = shift if scalar @_;
+		$self->{$name} = shift || 0 if scalar @_;
 		return $self->{$name};
 	};
 
@@ -1139,8 +1131,6 @@ sub AUTOLOAD {				## Default method
 
 1;
 
-__END__
-
 
 =head1 NAME
 
@@ -1161,13 +1151,15 @@ for all your resolving needs.
 
 =head1 METHODS
 
-=head2 new, domain, searchlist, nameservers, print, string, errorstring,
+=head2 new, domain, searchlist, nameserver, nameservers,
 
-=head2 search, query, send, bgsend, bgbusy, bgread, axfr, answerfrom,
+=head2 search, query, send, bgsend, bgbusy, bgread, axfr,
 
 =head2 force_v4, force_v6, prefer_v4, prefer_v6,
 
-=head2 dnssec, srcaddr, tsig, udppacketsize
+=head2 dnssec, srcaddr, tsig, udppacketsize,
+
+=head2 print, string, errorstring, replyfrom
 
 See L<Net::DNS::Resolver>.
 
@@ -1178,7 +1170,7 @@ Copyright (c)2003,2004 Chris Reinhardt.
 
 Portions Copyright (c)2005 Olaf Kolkman.
 
-Portions Copyright (c)2014,2015 Dick Franks.
+Portions Copyright (c)2014-2017 Dick Franks.
 
 All rights reserved.
 
@@ -1208,3 +1200,69 @@ L<perl>, L<Net::DNS>, L<Net::DNS::Resolver>
 
 =cut
 
+
+########################################
+
+__DATA__	## DEFAULT HINTS
+
+; <<>> DiG 9.11.4-RedHat-9.11.4-4.fc28 <<>> @b.root-servers.net . -t NS
+; (2 servers found)
+;; global options: +cmd
+;; Got answer:
+;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 44111
+;; flags: qr aa rd; QUERY: 1, ANSWER: 13, AUTHORITY: 0, ADDITIONAL: 27
+;; WARNING: recursion requested but not available
+
+;; OPT PSEUDOSECTION:
+; EDNS: version: 0, flags:; udp: 4096
+;; QUESTION SECTION:
+;.				IN	NS
+
+;; ANSWER SECTION:
+.			518400	IN	NS	c.root-servers.net.
+.			518400	IN	NS	k.root-servers.net.
+.			518400	IN	NS	l.root-servers.net.
+.			518400	IN	NS	j.root-servers.net.
+.			518400	IN	NS	b.root-servers.net.
+.			518400	IN	NS	g.root-servers.net.
+.			518400	IN	NS	h.root-servers.net.
+.			518400	IN	NS	d.root-servers.net.
+.			518400	IN	NS	a.root-servers.net.
+.			518400	IN	NS	f.root-servers.net.
+.			518400	IN	NS	i.root-servers.net.
+.			518400	IN	NS	m.root-servers.net.
+.			518400	IN	NS	e.root-servers.net.
+
+;; ADDITIONAL SECTION:
+a.root-servers.net.	3600000	IN	A	198.41.0.4
+b.root-servers.net.	3600000	IN	A	199.9.14.201
+c.root-servers.net.	3600000	IN	A	192.33.4.12
+d.root-servers.net.	3600000	IN	A	199.7.91.13
+e.root-servers.net.	3600000	IN	A	192.203.230.10
+f.root-servers.net.	3600000	IN	A	192.5.5.241
+g.root-servers.net.	3600000	IN	A	192.112.36.4
+h.root-servers.net.	3600000	IN	A	198.97.190.53
+i.root-servers.net.	3600000	IN	A	192.36.148.17
+j.root-servers.net.	3600000	IN	A	192.58.128.30
+k.root-servers.net.	3600000	IN	A	193.0.14.129
+l.root-servers.net.	3600000	IN	A	199.7.83.42
+m.root-servers.net.	3600000	IN	A	202.12.27.33
+a.root-servers.net.	3600000	IN	AAAA	2001:503:ba3e::2:30
+b.root-servers.net.	3600000	IN	AAAA	2001:500:200::b
+c.root-servers.net.	3600000	IN	AAAA	2001:500:2::c
+d.root-servers.net.	3600000	IN	AAAA	2001:500:2d::d
+e.root-servers.net.	3600000	IN	AAAA	2001:500:a8::e
+f.root-servers.net.	3600000	IN	AAAA	2001:500:2f::f
+g.root-servers.net.	3600000	IN	AAAA	2001:500:12::d0d
+h.root-servers.net.	3600000	IN	AAAA	2001:500:1::53
+i.root-servers.net.	3600000	IN	AAAA	2001:7fe::53
+j.root-servers.net.	3600000	IN	AAAA	2001:503:c27::2:30
+k.root-servers.net.	3600000	IN	AAAA	2001:7fd::1
+l.root-servers.net.	3600000	IN	AAAA	2001:500:9f::42
+m.root-servers.net.	3600000	IN	AAAA	2001:dc3::35
+
+;; Query time: 173 msec
+;; SERVER: 199.9.14.201#53(199.9.14.201)
+;; WHEN: Fri Aug 10 19:03:11 BST 2018
+;; MSG SIZE  rcvd: 811
+
