@@ -1,10 +1,9 @@
 package Net::DNS::Update;
 
 #
-# $Id: Update.pm 1455 2016-02-15 15:51:21Z willem $
+# $Id: Update.pm 1726 2018-12-15 12:59:56Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1455 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1726 $)[1];
 
 
 =head1 NAME
@@ -17,8 +16,8 @@ Net::DNS::Update - DNS dynamic update packet
 
     $update = new Net::DNS::Update( 'example.com', 'IN' );
 
-    $update->push( prereq => nxrrset('foo.example.com. A') );
-    $update->push( update => rr_add('foo.example.com. 86400 A 192.168.1.2') );
+    $update->push( prereq => nxrrset('foo.example.com. AAAA') );
+    $update->push( update => rr_add('foo.example.com. 86400 AAAA 2001::DB8::1') );
 
 =head1 DESCRIPTION
 
@@ -31,10 +30,14 @@ Programmers should refer to RFC2136 for dynamic update semantics.
 
 
 use strict;
-use base 'Net::DNS::Packet';
-
+use warnings;
+use integer;
 use Carp;
 
+use base qw(Net::DNS::Packet);
+
+use Net::DNS::Resolver;
+
 
 =head1 METHODS
 
@@ -49,8 +52,8 @@ dynamic update.	 Specifically, it creates a packet with the header
 opcode set to UPDATE and the zone record type to SOA (per RFC 2136,
 Section 2.3).
 
-Programs must use the push() method to add RRs to the prerequisite,
-update, and additional sections before performing the update.
+Programs must use the push() method to add RRs to the prerequisite
+and update sections before performing the update.
 
 Arguments are the zone name and the class.  The zone and class may
 be undefined or omitted and default to the default domain from the
@@ -62,15 +65,12 @@ sub new {
 	shift;
 	my ( $zone, @class ) = @_;
 
-	unless ( defined $zone ) {
-		require Net::DNS::Resolver;
-		($zone) = new Net::DNS::Resolver()->domain;	# default from resolver config
-	}
+	my ($domain) = grep defined && length, $zone, Net::DNS::Resolver->searchlist;
 
 	eval {
 		local $SIG{__DIE__};
 
-		my $self = __PACKAGE__->SUPER::new( $zone, 'SOA', @class );
+		my $self = __PACKAGE__->SUPER::new( $domain, 'SOA', @class );
 
 		my $header = $self->header;
 		$header->opcode('UPDATE');
@@ -167,12 +167,13 @@ the corresponding ( name => value ) form may also be used.
     # Create the update packet.
     my $update = new Net::DNS::Update('example.com');
 
-    # Prerequisite is that no A records exist for the name.
+    # Prerequisite is that no address records exist for the name.
     $update->push( pre => nxrrset('foo.example.com. A') );
+    $update->push( pre => nxrrset('foo.example.com. AAAA') );
 
-    # Add two A records for the name.
-    $update->push( update => rr_add('foo.example.com. 86400 A 192.168.1.2') );
-    $update->push( update => rr_add('foo.example.com. 86400 A 172.16.3.4') );
+    # Add two address records for the name.
+    $update->push( update => rr_add('foo.example.com. 86400 A 192.0.2.1') );
+    $update->push( update => rr_add('foo.example.com. 86400 AAAA 2001:DB8::1') );
 
     # Send the update to the zone's primary master.
     my $resolver = new Net::DNS::Resolver;
@@ -219,7 +220,7 @@ the corresponding ( name => value ) form may also be used.
 =head2 Perform a DNS update signed using a BIND private key file
 
     my $update = new Net::DNS::Update('example.com');
-    $update->push( update => rr_add('foo.example.com A 10.1.2.3') );
+    $update->push( update => rr_add('foo.example.com AAAA 2001:DB8::1') );
     $update->sign_tsig( "$dir/Khmac-sha512.example.com.+165+01018.private" );
     my $reply = $resolver->send( $update );
     $reply->verify( $update ) || die $reply->verifyerr;
@@ -243,7 +244,7 @@ the corresponding ( name => value ) form may also be used.
     $tsig->fudge(60);
 
     my $update = new Net::DNS::Update('example.com');
-    $update->push( update     => rr_add('foo.example.com A 10.1.2.3') );
+    $update->push( update     => rr_add('foo.example.com AAAA 2001:DB8::1') );
     $update->push( additional => $tsig );
 
 
